@@ -165,7 +165,7 @@ void do_health_check_child(int *node_id)
 
 		CHECK_REQUEST;
 
-		if (pool_config->health_check_period <= 0)
+		if (pool_config->health_check_params[*node_id].health_check_period <= 0)
 		{
 			sleep(30);
 		}
@@ -174,9 +174,10 @@ void do_health_check_child(int *node_id)
 		 * If health checking is enabled and the node is not in down status,
 		 * do health check.
 		 */
-		else if (pool_config->health_check_period > 0)
+		else if (pool_config->health_check_params[*node_id].health_check_period > 0)
 		{
 			bool result;
+			BackendInfo *bkinfo = pool_get_node_info(*node_id);
 
 			result = establish_persistent_connection(*node_id);
 
@@ -199,15 +200,31 @@ void do_health_check_child(int *node_id)
 					ereport(LOG, (errmsg("health check failed on node %d (timeout:%d)",
 										 *node_id, health_check_timer_expired)));
 
-					/* trigger failover */
-					partial = health_check_timer_expired?false:true;
-					degenerate_backend_set(node_id, 1, partial?REQ_DETAIL_SWITCHOVER:0);
+					if (bkinfo->backend_status == CON_DOWN && bkinfo->quarantine == true)
+					{
+						ereport(LOG, (errmsg("health check failed on quarantine node %d (timeout:%d)",
+											 *node_id, health_check_timer_expired),
+									  errdetail("ignoring..")));
+					}
+					else
+					{
+						/* trigger failover */
+						partial = health_check_timer_expired ? false : true;
+						degenerate_backend_set(node_id, 1, partial ? REQ_DETAIL_SWITCHOVER : 0);
+					}
 				}
 			}
+			else if (bkinfo->backend_status == CON_DOWN && bkinfo->quarantine == true)
+			{
+				/* The node has become reachable again. Reset
+				 * the quarantine state
+				 */
+				send_failback_request(*node_id, false, REQ_DETAIL_UPDATE | REQ_DETAIL_WATCHDOG);
+			}
 
 			/* Discard persistent connections */
 			discard_persistent_connection(*node_id);
-			sleep(pool_config->health_check_period);
+			sleep(pool_config->health_check_params[*node_id].health_check_period);
 		}
 	}
 	exit(0);
@@ -225,25 +242,27 @@ static bool establish_persistent_connection(int node)
 	bkinfo = pool_get_node_info(node);
 
 	/*
-	 * If the node is already in down status or unused,
-	 * do nothing.
+	 * If the node is already in down status or unused, do nothing.
+	 * except when the node state is down because of quarantine operation
+	 * since we want to detect when the node cames back to life again to
+	 * remove it from the quarantine state
 	 */
 	if (bkinfo->backend_status == CON_UNUSED ||
-		bkinfo->backend_status == CON_DOWN)
+		(bkinfo->backend_status == CON_DOWN && bkinfo->quarantine == false))
 		return false;
 
 	/*
 	 * If database is not specified, "postgres" database is assumed.
 	 */
-	if (*pool_config->health_check_database == '\0')
-		pool_config->health_check_database = "postgres";
+	if (*pool_config->health_check_params[node].health_check_database == '\0')
+		pool_config->health_check_params[node].health_check_database = "postgres";
 
 	/*
 	 * Try to connect to the database.
 	 */
 	if (slot == NULL)
 	{
-		retry_cnt = pool_config->health_check_max_retries;
+		retry_cnt = pool_config->health_check_params[node].health_check_max_retries;
 
 		do
 		{
@@ -252,22 +271,22 @@ static bool establish_persistent_connection(int node)
 			 * communication path failure much earlier before
 			 * TCP/IP stack detects it.
 			 */
-			if (pool_config->health_check_timeout > 0)
+			if (pool_config->health_check_params[node].health_check_timeout > 0)
 			{
 				CLEAR_ALARM;
 				pool_signal(SIGALRM, health_check_timer_handler);
-				alarm(pool_config->health_check_timeout);
+				alarm(pool_config->health_check_params[node].health_check_timeout);
 				errno = 0;
 				health_check_timer_expired = 0;
 			}
 
 			slot = make_persistent_db_connection_noerror(node, bkinfo->backend_hostname,
 														 bkinfo->backend_port,
-														 pool_config->health_check_database,
-														 pool_config->health_check_user,
-														 pool_config->health_check_password, false);
+														 pool_config->health_check_params[node].health_check_database,
+														 pool_config->health_check_params[node].health_check_user,
+														 pool_config->health_check_params[node].health_check_password, false);
 
-			if (pool_config->health_check_timeout > 0)
+			if (pool_config->health_check_params[node].health_check_timeout > 0)
 			{
 				/* cancel health check timer */
 				pool_signal(SIGALRM, SIG_IGN);
@@ -276,7 +295,7 @@ static bool establish_persistent_connection(int node)
 
 			if (slot)
 			{
-				if (retry_cnt != pool_config->health_check_max_retries)
+				if (retry_cnt != pool_config->health_check_params[node].health_check_max_retries)
 				{
 					ereport(LOG,
 							(errmsg("health check retrying on DB node: %d succeeded",
@@ -292,9 +311,9 @@ static bool establish_persistent_connection(int node)
 				ereport(LOG,
 						(errmsg("health check retrying on DB node: %d (round:%d)",
 								node,
-								pool_config->health_check_max_retries - retry_cnt)));
+								pool_config->health_check_params[node].health_check_max_retries - retry_cnt)));
 
-				sleep(pool_config->health_check_retry_delay);
+				sleep(pool_config->health_check_params[node].health_check_retry_delay);
 			}
 		} while (retry_cnt >= 0);
 	}
